package router

import (
	"html/template"
	"net/http"
	"os"

	"go.jolheiser.com/blog/markdown"
	"go.jolheiser.com/blog/post"
	"go.jolheiser.com/blog/static"

	"github.com/go-chi/chi/v5"
	"github.com/go-chi/chi/v5/middleware"
	"github.com/rs/zerolog/log"
)

func New(blog *post.Blog) *chi.Mux {
	m := chi.NewMux()

	m.Use(middleware.Logger)
	m.Use(middleware.Recoverer)

	m.Get("/", indexHandler(blog))
	m.Get("/{post}", fileHandler(blog))
	m.Get("/sakura.css", static.SakuraCSS)

	return m
}

func indexHandler(blog *post.Blog) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		if err := static.IndexTemplate.Execute(w, map[string]interface{}{
			"Blog": blog,
		}); err != nil {
			log.Error().Err(err).Msg("could not execute template")
		}
	}
}

func fileHandler(blog *post.Blog) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		postName := chi.URLParam(r, "post")
		p, ok := blog.Posts[postName]
		if !ok {
			w.WriteHeader(http.StatusNotFound)
			return
		}
		fi, err := os.Open(p.Path)
		if err != nil {
			log.Error().Err(err).Msg("could not open post")
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}
		defer fi.Close()

		md, err := markdown.Convert(fi)
		if err != nil {
			log.Error().Err(err).Msg("could not convert")
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}

		if err := static.PostTemplate.Execute(w, map[string]interface{}{
			"Post":    p,
			"Content": template.HTML(md),
		}); err != nil {
			log.Error().Err(err).Msg("could not execute template")
		}
	}
}
